@@ -28,6 +28,8 @@ BAZEL_VERSION = "6.3.2"
 
 PYARROW_VERSION = "14.0.0"
 
+NUMPY_VERSION = "1.26.4"
+
 PROJECT_ROOT_DIR = os.path.join(os.path.dirname(os.path.abspath(__file__)), "../")
 
 logging.basicConfig(
@@ -115,7 +117,7 @@ def _run_js():
 def _install_cpp_deps():
     _exec_cmd(f"pip install pyarrow=={PYARROW_VERSION}")
     _exec_cmd("pip install psutil")
-    _exec_cmd("pip install 'numpy<2.0.0'")
+    _exec_cmd(f"pip install numpy=={NUMPY_VERSION}")
     _install_bazel()
 
 
