@@ -21,10 +21,13 @@ package org.apache.fury;
 
 import java.io.OutputStream;
 import java.nio.ByteBuffer;
+import java.util.Collections;
+import java.util.Map;
 import java.util.WeakHashMap;
 import java.util.function.Consumer;
 import java.util.function.Function;
 import javax.annotation.concurrent.ThreadSafe;
+import org.apache.fury.annotation.Internal;
 import org.apache.fury.io.FuryInputStream;
 import org.apache.fury.io.FuryReadableChannel;
 import org.apache.fury.memory.MemoryBuffer;
@@ -45,17 +48,23 @@ public class ThreadLocalFury extends AbstractThreadSafeFury {
 
   private final ThreadLocal<LoaderBinding> bindingThreadLocal;
   private Consumer<Fury> factoryCallback;
-  private final WeakHashMap<LoaderBinding, Object> allFury;
+  private final Map<LoaderBinding, Object> allFury;
+
+  private ClassLoader classLoader;
 
   public ThreadLocalFury(Function<ClassLoader, Fury> furyFactory) {
     factoryCallback = f -> {};
-    allFury = new WeakHashMap<>();
+    allFury = Collections.synchronizedMap(new WeakHashMap<>());
     bindingThreadLocal =
         ThreadLocal.withInitial(
             () -> {
               LoaderBinding binding = new LoaderBinding(furyFactory);
               binding.setBindingCallback(factoryCallback);
-              binding.setClassLoader(Thread.currentThread().getContextClassLoader());
+              ClassLoader cl =
+                  classLoader == null
+                      ? Thread.currentThread().getContextClassLoader()
+                      : classLoader;
+              binding.setClassLoader(cl);
               allFury.put(binding, null);
               return binding;
             });
@@ -66,8 +75,9 @@ public class ThreadLocalFury extends AbstractThreadSafeFury {
     Fury fury = bindingThreadLocal.get().get();
   }
 
+  @Internal
   @Override
-  protected void processCallback(Consumer<Fury> callback) {
+  public void registerCallback(Consumer<Fury> callback) {
     factoryCallback = factoryCallback.andThen(callback);
     for (LoaderBinding binding : allFury.keySet()) {
       binding.visitAllFury(callback);
@@ -254,6 +264,7 @@ public class ThreadLocalFury extends AbstractThreadSafeFury {
 
   @Override
   public void setClassLoader(ClassLoader classLoader, StagingType stagingType) {
+    this.classLoader = classLoader;
     bindingThreadLocal.get().setClassLoader(classLoader, stagingType);
   }
 
