@@ -19,8 +19,9 @@
 
 package org.apache.fury.serializer;
 
-import static org.apache.fury.type.TypeUtils.PRIMITIVE_CHAR_ARRAY_TYPE;
 import static org.apache.fury.type.TypeUtils.STRING_TYPE;
+import static org.apache.fury.util.StringUtils.MULTI_CHARS_NON_ASCII_MASK;
+import static org.apache.fury.util.StringUtils.MULTI_CHARS_NON_LATIN_MASK;
 
 import java.lang.invoke.CallSite;
 import java.lang.invoke.LambdaMetafactory;
@@ -43,6 +44,7 @@ import org.apache.fury.reflect.ReflectionUtils;
 import org.apache.fury.type.Type;
 import org.apache.fury.util.MathUtils;
 import org.apache.fury.util.Preconditions;
+import org.apache.fury.util.StringEncodingUtils;
 import org.apache.fury.util.StringUtils;
 import org.apache.fury.util.unsafe._JDKAccess;
 
@@ -149,15 +151,19 @@ public final class StringSerializer extends ImmutableSerializer<String> {
   public Expression writeStringExpr(Expression strSerializer, Expression buffer, Expression str) {
     if (isJava) {
       if (STRING_VALUE_FIELD_IS_BYTES) {
-        return new StaticInvoke(StringSerializer.class, "writeBytesString", buffer, str);
+        if (compressString) {
+          return new Invoke(strSerializer, "writeCompressedBytesString", buffer, str);
+        } else {
+          return new StaticInvoke(StringSerializer.class, "writeBytesString", buffer, str);
+        }
       } else {
         if (!STRING_VALUE_FIELD_IS_CHARS) {
           throw new UnsupportedOperationException();
         }
         if (compressString) {
-          return new Invoke(strSerializer, "writeCharsStringCompressed", buffer, str);
+          return new Invoke(strSerializer, "writeCompressedCharsString", buffer, str);
         } else {
-          return new Invoke(strSerializer, "writeCharsStringUncompressed", buffer, str);
+          return new Invoke(strSerializer, "writeCharsString", buffer, str);
         }
       }
     } else {
@@ -165,23 +171,6 @@ public final class StringSerializer extends ImmutableSerializer<String> {
     }
   }
 
-  // Invoked by jit
-  public void writeCharsStringCompressed(MemoryBuffer buffer, String value) {
-    final char[] chars = (char[]) Platform.getObject(value, STRING_VALUE_FIELD_OFFSET);
-    if (StringUtils.isLatin(chars)) {
-      writeCharsLatin(buffer, chars, chars.length);
-    } else {
-      writeCharsUTF16(buffer, chars, chars.length);
-    }
-  }
-
-  // Invoked by jit
-  public void writeCharsStringUncompressed(MemoryBuffer buffer, String value) {
-    int numBytes = MathUtils.doubleExact(value.length());
-    final char[] chars = (char[]) Platform.getObject(value, STRING_VALUE_FIELD_OFFSET);
-    buffer.writePrimitiveArrayWithSize(chars, Platform.CHAR_ARRAY_OFFSET, numBytes);
-  }
-
   public String readString(MemoryBuffer buffer) {
     if (isJava) {
       return readJavaString(buffer);
@@ -201,9 +190,7 @@ public final class StringSerializer extends ImmutableSerializer<String> {
         if (compressString) {
           return new Invoke(strSerializer, "readCompressedCharsString", STRING_TYPE, buffer);
         } else {
-          Expression chars = new Invoke(buffer, "readCharsAndSize", PRIMITIVE_CHAR_ARRAY_TYPE);
-          return new StaticInvoke(
-              StringSerializer.class, "newCharsStringZeroCopy", STRING_TYPE, chars);
+          return new Invoke(strSerializer, "readCharsString", STRING_TYPE, buffer);
         }
       }
     } else {
@@ -216,17 +203,7 @@ public final class StringSerializer extends ImmutableSerializer<String> {
     long header = buffer.readVarUint36Small();
     byte coder = (byte) (header & 0b11);
     int numBytes = (int) (header >>> 2);
-    buffer.checkReadableBytes(numBytes);
-    byte[] bytes;
-    byte[] heapMemory = buffer.getHeapMemory();
-    if (heapMemory != null) {
-      final int arrIndex = buffer._unsafeHeapReaderIndex();
-      buffer.increaseReaderIndex(numBytes);
-      bytes = new byte[numBytes];
-      System.arraycopy(heapMemory, arrIndex, bytes, 0, numBytes);
-    } else {
-      bytes = buffer.readBytes(numBytes);
-    }
+    byte[] bytes = readBytesUnCompressedUTF16(buffer, numBytes);
     if (coder != UTF8) {
       return newBytesStringZeroCopy(coder, bytes);
     } else {
@@ -234,81 +211,131 @@ public final class StringSerializer extends ImmutableSerializer<String> {
     }
   }
 
+  @CodegenInvoke
+  public String readCharsString(MemoryBuffer buffer) {
+    long header = buffer.readVarUint36Small();
+    byte coder = (byte) (header & 0b11);
+    int numBytes = (int) (header >>> 2);
+    char[] chars;
+    if (coder == LATIN1) {
+      chars = readCharsLatin1(buffer, numBytes);
+    } else if (coder == UTF16) {
+      chars = readCharsUTF16(buffer, numBytes);
+    } else {
+      throw new RuntimeException("Unknown coder type " + coder);
+    }
+    return newCharsStringZeroCopy(chars);
+  }
+
+  @CodegenInvoke
+  public String readCompressedBytesString(MemoryBuffer buffer) {
+    long header = buffer.readVarUint36Small();
+    byte coder = (byte) (header & 0b11);
+    int numBytes = (int) (header >>> 2);
+    if (coder == UTF8) {
+      return newBytesStringZeroCopy(UTF16, readBytesUTF8(buffer, numBytes));
+    } else if (coder == LATIN1 || coder == UTF16) {
+      return newBytesStringZeroCopy(coder, readBytesUnCompressedUTF16(buffer, numBytes));
+    } else {
+      throw new RuntimeException("Unknown coder type " + coder);
+    }
+  }
+
   @CodegenInvoke
   public String readCompressedCharsString(MemoryBuffer buffer) {
     long header = buffer.readVarUint36Small();
     byte coder = (byte) (header & 0b11);
     int numBytes = (int) (header >>> 2);
+    char[] chars;
     if (coder == LATIN1) {
-      return newCharsStringZeroCopy(readLatinChars(buffer, numBytes));
+      chars = readCharsLatin1(buffer, numBytes);
+    } else if (coder == UTF8) {
+      chars = readCharsUTF8(buffer, numBytes);
     } else if (coder == UTF16) {
-      return newCharsStringZeroCopy(readUTF16Chars(buffer, numBytes));
+      chars = readCharsUTF16(buffer, numBytes);
     } else {
-      return readUtf8(buffer, coder, numBytes);
+      throw new RuntimeException("Unknown coder type " + coder);
     }
-  }
-
-  private String readUtf8(MemoryBuffer buffer, byte coder, int numBytes) {
-    Preconditions.checkArgument(coder == UTF8, UTF8);
-    byte[] bytes = buffer.readBytes(numBytes);
-    return new String(bytes, 0, numBytes, StandardCharsets.UTF_8);
-  }
-
-  private byte[] getByteArray(int numElements) {
-    byte[] byteArray = this.byteArray;
-    if (byteArray.length < numElements) {
-      byteArray = new byte[numElements];
-      this.byteArray = byteArray;
-    }
-    if (byteArray.length > DEFAULT_BUFFER_SIZE) {
-      smoothByteArrayLength =
-          Math.max(((int) (smoothByteArrayLength * 0.9 + numElements * 0.1)), DEFAULT_BUFFER_SIZE);
-      if (smoothByteArrayLength <= DEFAULT_BUFFER_SIZE) {
-        this.byteArray = new byte[DEFAULT_BUFFER_SIZE];
-      }
-    }
-    return byteArray;
+    return newCharsStringZeroCopy(chars);
   }
 
   // Invoked by fury JIT
   public void writeJavaString(MemoryBuffer buffer, String value) {
     if (STRING_VALUE_FIELD_IS_BYTES) {
-      writeBytesString(buffer, value);
+      if (compressString) {
+        writeCompressedBytesString(buffer, value);
+      } else {
+        writeBytesString(buffer, value);
+      }
     } else {
       assert STRING_VALUE_FIELD_IS_CHARS;
-      final char[] chars = (char[]) Platform.getObject(value, STRING_VALUE_FIELD_OFFSET);
       if (compressString) {
-        if (StringUtils.isLatin(chars)) {
-          writeCharsLatin(buffer, chars, chars.length);
-        } else {
-          writeCharsUTF16(buffer, chars, chars.length);
-        }
+        writeCompressedCharsString(buffer, value);
       } else {
-        int numBytes = MathUtils.doubleExact(value.length());
-        buffer.writePrimitiveArrayWithSize(chars, Platform.CHAR_ARRAY_OFFSET, numBytes);
+        writeCharsString(buffer, value);
       }
     }
   }
 
+  @CodegenInvoke
+  public void writeUTF8String(MemoryBuffer buffer, String value) {
+    byte[] bytes = value.getBytes(StandardCharsets.UTF_8);
+    buffer.writeVarUint32(bytes.length);
+    buffer.writeBytes(bytes);
+  }
+
   // Invoked by fury JIT
   public String readJavaString(MemoryBuffer buffer) {
     if (STRING_VALUE_FIELD_IS_BYTES) {
-      return readBytesString(buffer);
+      if (compressString) {
+        return readCompressedBytesString(buffer);
+      } else {
+        return readBytesString(buffer);
+      }
     } else {
       assert STRING_VALUE_FIELD_IS_CHARS;
       if (compressString) {
         return readCompressedCharsString(buffer);
       } else {
-        return newCharsStringZeroCopy(buffer.readCharsAndSize());
+        return readCharsString(buffer);
       }
     }
   }
 
+  @CodegenInvoke
+  public void writeCompressedBytesString(MemoryBuffer buffer, String value) {
+    final byte[] bytes = (byte[]) Platform.getObject(value, STRING_VALUE_FIELD_OFFSET);
+    final byte coder = Platform.getByte(value, Offset.STRING_CODER_FIELD_OFFSET);
+    if (coder == LATIN1 || bestCoder(bytes) == UTF16) {
+      writeBytesString(buffer, coder, bytes);
+    } else {
+      writeBytesUTF8(buffer, bytes);
+    }
+  }
+
+  @CodegenInvoke
+  public void writeCompressedCharsString(MemoryBuffer buffer, String value) {
+    final char[] chars = (char[]) Platform.getObject(value, STRING_VALUE_FIELD_OFFSET);
+    final byte coder = bestCoder(chars);
+    if (coder == LATIN1) {
+      writeCharsLatin1(buffer, chars, chars.length);
+    } else if (coder == UTF8) {
+      writeCharsUTF8(buffer, chars);
+    } else {
+      writeCharsUTF16(buffer, chars, chars.length);
+    }
+  }
+
+  @CodegenInvoke
   public static void writeBytesString(MemoryBuffer buffer, String value) {
     byte[] bytes = (byte[]) Platform.getObject(value, STRING_VALUE_FIELD_OFFSET);
+    byte coder = Platform.getByte(value, Offset.STRING_CODER_FIELD_OFFSET);
+    writeBytesString(buffer, coder, bytes);
+  }
+
+  public static void writeBytesString(MemoryBuffer buffer, byte coder, byte[] bytes) {
     int bytesLen = bytes.length;
-    long header =
-        ((long) bytesLen << 2) | Platform.getByte(value, Offset.STRING_CODER_FIELD_OFFSET);
+    long header = ((long) bytesLen << 2) | coder;
     int writerIndex = buffer.writerIndex();
     // The `ensure` ensure next operations are safe without bound checks,
     // and inner heap buffer doesn't change.
@@ -332,112 +359,93 @@ public final class StringSerializer extends ImmutableSerializer<String> {
     buffer._unsafeWriterIndex(writerIndex);
   }
 
-  public void writeCharsLatin(MemoryBuffer buffer, char[] chars, final int strLen) {
-    int writerIndex = buffer.writerIndex();
-    // The `ensure` ensure next operations are safe without bound checks,
-    // and inner heap buffer doesn't change.
-    buffer.ensure(writerIndex + 9 + strLen);
-    long header = ((long) strLen << 2) | LATIN1;
+  @CodegenInvoke
+  public void writeCharsString(MemoryBuffer buffer, String value) {
+    final char[] chars = (char[]) Platform.getObject(value, STRING_VALUE_FIELD_OFFSET);
+    if (StringUtils.isLatin(chars)) {
+      writeCharsLatin1(buffer, chars, chars.length);
+    } else {
+      writeCharsUTF16(buffer, chars, chars.length);
+    }
+  }
+
+  @CodegenInvoke
+  public String readUTF8String(MemoryBuffer buffer) {
+    int numBytes = buffer.readVarUint32Small14();
+    buffer.checkReadableBytes(numBytes);
     final byte[] targetArray = buffer.getHeapMemory();
     if (targetArray != null) {
-      int arrIndex = buffer._unsafeHeapWriterIndex();
-      int written = LittleEndian.putVarUint36Small(targetArray, arrIndex, header);
-      arrIndex += written;
-      writerIndex += written + strLen;
-      for (int i = 0; i < strLen; i++) {
-        targetArray[arrIndex + i] = (byte) chars[i];
-      }
-      buffer._unsafeWriterIndex(writerIndex);
+      String str =
+          new String(
+              targetArray, buffer._unsafeHeapReaderIndex(), numBytes, StandardCharsets.UTF_8);
+      buffer.increaseReaderIndex(numBytes);
+      return str;
     } else {
-      writerIndex += buffer._unsafePutVarUint36Small(writerIndex, header);
-      final byte[] tmpArray = getByteArray(strLen);
-      // Write to heap memory then copy is 60% faster than unsafe write to direct memory.
-      for (int i = 0; i < strLen; i++) {
-        tmpArray[i] = (byte) chars[i];
-      }
-      buffer.put(writerIndex, tmpArray, 0, strLen);
-      writerIndex += strLen;
-      buffer._unsafeWriterIndex(writerIndex);
+      final byte[] tmpArray = getByteArray(numBytes);
+      buffer.readBytes(tmpArray, 0, numBytes);
+      return new String(tmpArray, 0, numBytes, StandardCharsets.UTF_8);
     }
   }
 
-  public void writeCharsUTF16(MemoryBuffer buffer, char[] chars, int strLen) {
-    int numBytes = MathUtils.doubleExact(strLen);
-    long header = ((long) numBytes << 2) | UTF16;
-    // The `ensure` ensure next operations are safe without bound checks,
-    // and inner heap buffer doesn't change.
-    int writerIndex = buffer.writerIndex();
-    buffer.ensure(writerIndex + 9 + numBytes);
-    byte[] targetArray = buffer.getHeapMemory();
-    if (targetArray != null) {
-      int arrIndex = buffer._unsafeHeapWriterIndex();
-      int written = LittleEndian.putVarUint36Small(targetArray, arrIndex, header);
-      arrIndex += written;
-      writerIndex += written + numBytes;
-      if (Platform.IS_LITTLE_ENDIAN) {
-        // FIXME JDK11 utf16 string uses little-endian order.
-        Platform.UNSAFE.copyMemory(
-            chars,
-            Platform.CHAR_ARRAY_OFFSET,
-            targetArray,
-            Platform.BYTE_ARRAY_OFFSET + arrIndex,
-            numBytes);
-      } else {
-        heapWriteCharsUTF16BE(chars, arrIndex, numBytes, targetArray);
-      }
-    } else {
-      writerIndex = offHeapWriteCharsUTF16(buffer, chars, writerIndex, header, numBytes);
-    }
-    buffer._unsafeWriterIndex(writerIndex);
-  }
-
-  private static void heapWriteCharsUTF16BE(
-      char[] chars, int arrIndex, int numBytes, byte[] targetArray) {
-    // Write to heap memory then copy is 250% faster than unsafe write to direct memory.
-    int charIndex = 0;
-    for (int i = arrIndex, end = i + numBytes; i < end; i += 2) {
-      char c = chars[charIndex++];
-      targetArray[i] = (byte) (c >> StringUTF16.HI_BYTE_SHIFT);
-      targetArray[i + 1] = (byte) (c >> StringUTF16.LO_BYTE_SHIFT);
-    }
-  }
-
-  private int offHeapWriteCharsUTF16(
-      MemoryBuffer buffer, char[] chars, int writerIndex, long header, int numBytes) {
-    writerIndex += buffer._unsafePutVarUint36Small(writerIndex, header);
-    byte[] tmpArray = getByteArray(numBytes);
-    int charIndex = 0;
-    for (int i = 0; i < numBytes; i += 2) {
-      char c = chars[charIndex++];
-      tmpArray[i] = (byte) (c >> StringUTF16.HI_BYTE_SHIFT);
-      tmpArray[i + 1] = (byte) (c >> StringUTF16.LO_BYTE_SHIFT);
-    }
-    buffer.put(writerIndex, tmpArray, 0, numBytes);
-    writerIndex += numBytes;
-    return writerIndex;
-  }
-
-  private char[] readLatinChars(MemoryBuffer buffer, int numBytes) {
-    char[] chars = new char[numBytes];
+  public char[] readCharsLatin1(MemoryBuffer buffer, int numBytes) {
     buffer.checkReadableBytes(numBytes);
-    byte[] targetArray = buffer.getHeapMemory();
-    if (targetArray != null) {
+    byte[] srcArray = buffer.getHeapMemory();
+    char[] chars = new char[numBytes];
+    if (srcArray != null) {
       int srcIndex = buffer._unsafeHeapReaderIndex();
       for (int i = 0; i < numBytes; i++) {
-        chars[i] = (char) (targetArray[srcIndex++] & 0xff);
+        chars[i] = (char) (srcArray[srcIndex++] & 0xff);
       }
       buffer._increaseReaderIndexUnsafe(numBytes);
     } else {
-      byte[] byteArray = getByteArray(numBytes);
-      buffer.readBytes(byteArray, 0, numBytes);
+      byte[] tmpArray = getByteArray(numBytes);
+      buffer.readBytes(tmpArray, 0, numBytes);
       for (int i = 0; i < numBytes; i++) {
-        chars[i] = (char) (byteArray[i] & 0xff);
+        chars[i] = (char) (tmpArray[i] & 0xff);
       }
     }
     return chars;
   }
 
-  private char[] readUTF16Chars(MemoryBuffer buffer, int numBytes) {
+  public byte[] readBytesUTF8(MemoryBuffer buffer, int numBytes) {
+    int udf8Bytes = buffer.readInt32();
+    byte[] bytes = new byte[numBytes];
+    buffer.checkReadableBytes(udf8Bytes);
+    byte[] srcArray = buffer.getHeapMemory();
+    if (srcArray != null) {
+      int srcIndex = buffer._unsafeHeapReaderIndex();
+      int readLen = StringEncodingUtils.convertUTF8ToUTF16(srcArray, srcIndex, udf8Bytes, bytes);
+      if (readLen != numBytes) {
+        throw new RuntimeException("Decode UTF8 to UTF16 failed");
+      }
+      buffer._increaseReaderIndexUnsafe(udf8Bytes);
+    } else {
+      byte[] tmpArray = getByteArray(udf8Bytes);
+      buffer.readBytes(tmpArray, 0, udf8Bytes);
+      int readLen = StringEncodingUtils.convertUTF8ToUTF16(tmpArray, 0, udf8Bytes, bytes);
+      if (readLen != numBytes) {
+        throw new RuntimeException("Decode UTF8 to UTF16 failed");
+      }
+    }
+    return bytes;
+  }
+
+  public byte[] readBytesUnCompressedUTF16(MemoryBuffer buffer, int numBytes) {
+    buffer.checkReadableBytes(numBytes);
+    byte[] bytes;
+    byte[] heapMemory = buffer.getHeapMemory();
+    if (heapMemory != null) {
+      final int arrIndex = buffer._unsafeHeapReaderIndex();
+      buffer.increaseReaderIndex(numBytes);
+      bytes = new byte[numBytes];
+      System.arraycopy(heapMemory, arrIndex, bytes, 0, numBytes);
+    } else {
+      bytes = buffer.readBytes(numBytes);
+    }
+    return bytes;
+  }
+
+  public char[] readCharsUTF16(MemoryBuffer buffer, int numBytes) {
     char[] chars = new char[numBytes >> 1];
     if (Platform.IS_LITTLE_ENDIAN) {
       // FIXME JDK11 utf16 string uses little-endian order.
@@ -471,6 +479,138 @@ public final class StringSerializer extends ImmutableSerializer<String> {
     return chars;
   }
 
+  public char[] readCharsUTF8(MemoryBuffer buffer, int numBytes) {
+    int udf16Chars = numBytes >> 1;
+    int udf8Bytes = buffer.readInt32();
+    char[] chars = new char[udf16Chars];
+    buffer.checkReadableBytes(udf8Bytes);
+    byte[] srcArray = buffer.getHeapMemory();
+    if (srcArray != null) {
+      int srcIndex = buffer._unsafeHeapReaderIndex();
+      int readLen = StringEncodingUtils.convertUTF8ToUTF16(srcArray, srcIndex, udf8Bytes, chars);
+      if (readLen != udf16Chars) {
+        throw new RuntimeException("Decode UTF8 to UTF16 failed");
+      }
+      buffer._increaseReaderIndexUnsafe(udf8Bytes);
+    } else {
+      byte[] tmpArray = getByteArray(udf8Bytes);
+      buffer.readBytes(tmpArray, 0, udf8Bytes);
+      int readLen = StringEncodingUtils.convertUTF8ToUTF16(tmpArray, 0, udf8Bytes, chars);
+      if (readLen != udf16Chars) {
+        throw new RuntimeException("Decode UTF8 to UTF16 failed");
+      }
+    }
+    return chars;
+  }
+
+  public void writeCharsLatin1(MemoryBuffer buffer, char[] chars, int numBytes) {
+    int writerIndex = buffer.writerIndex();
+    long header = ((long) numBytes << 2) | LATIN1;
+    buffer.ensure(writerIndex + 5 + numBytes);
+    byte[] targetArray = buffer.getHeapMemory();
+    if (targetArray != null) {
+      final int targetIndex = buffer._unsafeHeapWriterIndex();
+      int arrIndex = targetIndex;
+      arrIndex += LittleEndian.putVarUint36Small(targetArray, arrIndex, header);
+      writerIndex += arrIndex - targetIndex;
+      for (int i = 0; i < numBytes; i++) {
+        targetArray[arrIndex + i] = (byte) chars[i];
+      }
+    } else {
+      writerIndex += buffer._unsafePutVarUint36Small(writerIndex, header);
+      final byte[] tmpArray = getByteArray(numBytes);
+      for (int i = 0; i < numBytes; i++) {
+        tmpArray[i] = (byte) chars[i];
+      }
+      buffer.put(writerIndex, tmpArray, 0, numBytes);
+    }
+    writerIndex += numBytes;
+    buffer._unsafeWriterIndex(writerIndex);
+  }
+
+  public void writeCharsUTF16(MemoryBuffer buffer, char[] chars, int numChars) {
+    int numBytes = MathUtils.doubleExact(numChars);
+    int writerIndex = buffer.writerIndex();
+    long header = ((long) numBytes << 2) | UTF16;
+    buffer.ensure(writerIndex + 5 + numBytes);
+    final byte[] targetArray = buffer.getHeapMemory();
+    if (targetArray != null) {
+      final int targetIndex = buffer._unsafeHeapWriterIndex();
+      int arrIndex = targetIndex;
+      arrIndex += LittleEndian.putVarUint36Small(targetArray, arrIndex, header);
+      writerIndex += arrIndex - targetIndex + numBytes;
+      if (Platform.IS_LITTLE_ENDIAN) {
+        // FIXME JDK11 utf16 string uses little-endian order.
+        Platform.UNSAFE.copyMemory(
+            chars,
+            Platform.CHAR_ARRAY_OFFSET,
+            targetArray,
+            Platform.BYTE_ARRAY_OFFSET + arrIndex,
+            numBytes);
+      } else {
+        heapWriteCharsUTF16BE(chars, arrIndex, numBytes, targetArray);
+      }
+    } else {
+      writerIndex += buffer._unsafePutVarUint36Small(writerIndex, header);
+      writerIndex = offHeapWriteCharsUTF16(buffer, chars, writerIndex, numBytes);
+    }
+    buffer._unsafeWriterIndex(writerIndex);
+  }
+
+  public void writeCharsUTF8(MemoryBuffer buffer, char[] chars) {
+    int estimateMaxBytes = chars.length * 3;
+    int numBytes = MathUtils.doubleExact(chars.length);
+    int writerIndex = buffer.writerIndex();
+    long header = ((long) numBytes << 2) | UTF8;
+    buffer.ensure(writerIndex + 9 + estimateMaxBytes);
+    byte[] targetArray = buffer.getHeapMemory();
+    if (targetArray != null) {
+      int targetIndex = buffer._unsafeHeapWriterIndex();
+      int arrIndex = targetIndex;
+      arrIndex += LittleEndian.putVarUint36Small(targetArray, arrIndex, header);
+      writerIndex += arrIndex - targetIndex;
+      targetIndex = StringEncodingUtils.convertUTF16ToUTF8(chars, targetArray, arrIndex + 4);
+      int written = targetIndex - arrIndex - 4;
+      buffer._unsafePutInt32(writerIndex, written);
+      buffer._unsafeWriterIndex(writerIndex + 4 + written);
+    } else {
+      final byte[] tmpArray = getByteArray(estimateMaxBytes);
+      int written = StringEncodingUtils.convertUTF16ToUTF8(chars, tmpArray, 0);
+      writerIndex += buffer._unsafePutVarUint36Small(writerIndex, header);
+      buffer._unsafePutInt32(writerIndex, written);
+      writerIndex += 4;
+      buffer.put(writerIndex, tmpArray, 0, written);
+      buffer._unsafeWriterIndex(writerIndex + written);
+    }
+  }
+
+  public void writeBytesUTF8(MemoryBuffer buffer, byte[] bytes) {
+    int numBytes = bytes.length;
+    int estimateMaxBytes = bytes.length / 2 * 3;
+    int writerIndex = buffer.writerIndex();
+    long header = ((long) numBytes << 2) | UTF8;
+    buffer.ensure(writerIndex + 9 + estimateMaxBytes);
+    byte[] targetArray = buffer.getHeapMemory();
+    if (targetArray != null) {
+      int targetIndex = buffer._unsafeHeapWriterIndex();
+      int arrIndex = targetIndex;
+      arrIndex += LittleEndian.putVarUint36Small(targetArray, arrIndex, header);
+      writerIndex += arrIndex - targetIndex;
+      targetIndex = StringEncodingUtils.convertUTF16ToUTF8(bytes, targetArray, arrIndex + 4);
+      int written = targetIndex - arrIndex - 4;
+      buffer._unsafePutInt32(writerIndex, written);
+      buffer._unsafeWriterIndex(writerIndex + 4 + written);
+    } else {
+      final byte[] tmpArray = getByteArray(estimateMaxBytes);
+      int written = StringEncodingUtils.convertUTF16ToUTF8(bytes, tmpArray, 0);
+      writerIndex += buffer._unsafePutVarUint36Small(writerIndex, header);
+      buffer._unsafePutInt32(writerIndex, written);
+      writerIndex += 4;
+      buffer.put(writerIndex, tmpArray, 0, written);
+      buffer._unsafeWriterIndex(writerIndex + written);
+    }
+  }
+
   private static final MethodHandles.Lookup STRING_LOOK_UP =
       _JDKAccess._trustedLookup(String.class);
   private static final BiFunction<char[], Boolean, String> CHARS_STRING_ZERO_COPY_CTR =
@@ -603,26 +743,134 @@ public final class StringSerializer extends ImmutableSerializer<String> {
     }
   }
 
-  public void writeUTF8String(MemoryBuffer buffer, String value) {
-    byte[] bytes = value.getBytes(StandardCharsets.UTF_8);
-    buffer.writeVarUint32(bytes.length);
-    buffer.writeBytes(bytes);
-  }
-
-  public String readUTF8String(MemoryBuffer buffer) {
-    int numBytes = buffer.readVarUint32Small14();
-    buffer.checkReadableBytes(numBytes);
-    final byte[] targetArray = buffer.getHeapMemory();
-    if (targetArray != null) {
-      String str =
-          new String(
-              targetArray, buffer._unsafeHeapReaderIndex(), numBytes, StandardCharsets.UTF_8);
-      buffer.increaseReaderIndex(numBytes);
-      return str;
-    } else {
-      final byte[] tmpArray = getByteArray(numBytes);
-      buffer.readBytes(tmpArray, 0, numBytes);
-      return new String(tmpArray, 0, numBytes, StandardCharsets.UTF_8);
+  private static void heapWriteCharsUTF16BE(
+      char[] chars, int arrIndex, int numBytes, byte[] targetArray) {
+    // Write to heap memory then copy is 250% faster than unsafe write to direct memory.
+    int charIndex = 0;
+    for (int i = arrIndex, end = i + numBytes; i < end; i += 2) {
+      char c = chars[charIndex++];
+      targetArray[i] = (byte) (c >> StringUTF16.HI_BYTE_SHIFT);
+      targetArray[i + 1] = (byte) (c >> StringUTF16.LO_BYTE_SHIFT);
     }
   }
+
+  private int offHeapWriteCharsUTF16(
+      MemoryBuffer buffer, char[] chars, int writerIndex, int numBytes) {
+    byte[] tmpArray = getByteArray(numBytes);
+    int charIndex = 0;
+    for (int i = 0; i < numBytes; i += 2) {
+      char c = chars[charIndex++];
+      tmpArray[i] = (byte) (c >> StringUTF16.HI_BYTE_SHIFT);
+      tmpArray[i + 1] = (byte) (c >> StringUTF16.LO_BYTE_SHIFT);
+    }
+    buffer.put(writerIndex, tmpArray, 0, numBytes);
+    writerIndex += numBytes;
+    return writerIndex;
+  }
+
+  private static byte bestCoder(char[] chars) {
+    int numChars = chars.length;
+    // sample 64 chars
+    int sampleNum = Math.min(64, numChars);
+    int vectorizedLen = sampleNum >> 2;
+    int vectorizedChars = vectorizedLen << 2;
+    int endOffset = Platform.CHAR_ARRAY_OFFSET + (vectorizedChars << 1);
+    int asciiCount = 0;
+    int latin1Count = 0;
+    for (int offset = Platform.CHAR_ARRAY_OFFSET, charOffset = 0;
+        offset < endOffset;
+        offset += 8, charOffset += 4) {
+      long multiChars = Platform.getLong(chars, offset);
+      if ((multiChars & MULTI_CHARS_NON_ASCII_MASK) == 0) {
+        latin1Count += 4;
+        asciiCount += 4;
+      } else if ((multiChars & MULTI_CHARS_NON_LATIN_MASK) == 0) {
+        latin1Count += 4;
+        for (int i = 0; i < 4; ++i) {
+          if (chars[charOffset + i] < 0x80) {
+            asciiCount++;
+          }
+        }
+      } else {
+        for (int i = 0; i < 4; ++i) {
+          if (chars[charOffset + i] < 0x80) {
+            latin1Count++;
+            asciiCount++;
+          } else if (chars[charOffset + i] <= 0xFF) {
+            latin1Count++;
+          }
+        }
+      }
+    }
+
+    for (int i = vectorizedChars; i < sampleNum; i++) {
+      if (chars[i] < 0x80) {
+        latin1Count++;
+        asciiCount++;
+      } else if (chars[i] <= 0xFF) {
+        latin1Count++;
+      }
+    }
+
+    if (latin1Count == numChars
+        || (latin1Count == sampleNum && StringUtils.isLatin(chars, sampleNum))) {
+      return LATIN1;
+    } else if (asciiCount >= sampleNum * 0.5) {
+      // ascii number > 50%, choose UTF-8
+      return UTF8;
+    } else {
+      return UTF16;
+    }
+  }
+
+  private static byte bestCoder(byte[] bytes) {
+    int numBytes = bytes.length;
+    // sample 64 chars
+    int sampleNum = Math.min(64 << 1, numBytes);
+    int vectorizedLen = sampleNum >> 3;
+    int vectorizedBytes = vectorizedLen << 3;
+    int endOffset = Platform.BYTE_ARRAY_OFFSET + vectorizedBytes;
+    int asciiCount = 0;
+    for (int offset = Platform.BYTE_ARRAY_OFFSET, bytesOffset = 0;
+        offset < endOffset;
+        offset += 8, bytesOffset += 8) {
+      long multiChars = Platform.getLong(bytes, offset);
+      if ((multiChars & MULTI_CHARS_NON_ASCII_MASK) == 0) {
+        asciiCount += 4;
+      } else {
+        for (int i = 0; i < 8; i += 2) {
+          if (Platform.getChar(bytes, offset + i) < 0x80) {
+            asciiCount++;
+          }
+        }
+      }
+    }
+    for (int i = vectorizedBytes; vectorizedBytes < sampleNum; vectorizedBytes += 2) {
+      if (Platform.getChar(bytes, Platform.BYTE_ARRAY_OFFSET + i) < 0x80) {
+        asciiCount++;
+      }
+    }
+    // ascii number > 50%, choose UTF-8
+    if (asciiCount >= sampleNum * 0.5) {
+      return UTF8;
+    } else {
+      return UTF16;
+    }
+  }
+
+  private byte[] getByteArray(int numElements) {
+    byte[] byteArray = this.byteArray;
+    if (byteArray.length < numElements) {
+      byteArray = new byte[numElements];
+      this.byteArray = byteArray;
+    }
+    if (byteArray.length > DEFAULT_BUFFER_SIZE) {
+      smoothByteArrayLength =
+          Math.max(((int) (smoothByteArrayLength * 0.9 + numElements * 0.1)), DEFAULT_BUFFER_SIZE);
+      if (smoothByteArrayLength <= DEFAULT_BUFFER_SIZE) {
+        this.byteArray = new byte[DEFAULT_BUFFER_SIZE];
+      }
+    }
+    return byteArray;
+  }
 }
