@@ -21,13 +21,17 @@ package org.apache.fury.resolver;
 
 import static org.testng.Assert.*;
 
+import org.apache.fury.config.Config;
+import org.apache.fury.config.FuryBuilder;
 import org.testng.annotations.Test;
 
 public class SerializationContextTest {
 
   @Test
   public void testSerializationContext() {
-    SerializationContext context = new SerializationContext();
+    FuryBuilder builder = new FuryBuilder().withDeserializeNonexistentClass(false);
+    builder.build(); // trigger finish
+    SerializationContext context = new SerializationContext(new Config(builder));
     assertFalse(context.containsKey("A"));
     context.add("A", 1);
     assertTrue(context.containsKey("A"));
